@@ -149,10 +149,13 @@ typedef enum {
     OPTION_REVERSE_DDC,
     OPTION_LVDS_PROBE_PLL,
     OPTION_ACCELMETHOD,
-    OPTION_CONSTANTDPI,
     OPTION_CONNECTORTABLE,
     OPTION_DRI,
-    OPTION_DEFAULT_CONNECTOR_TABLE
+    OPTION_DEFAULT_CONNECTOR_TABLE,
+#if defined(__powerpc__)
+    OPTION_MAC_MODEL,
+#endif
+    OPTION_DEFAULT_TMDS_PLL
 } RADEONOpts;
 
 
@@ -422,6 +425,15 @@ typedef enum {
        CHIP_ERRATA_PLL_DELAY           = 0x00000004
 } RADEONErrata;
 
+#if defined(__powerpc__)
+typedef enum {
+       RADEON_MAC_IBOOK             = 0x00000001,
+       RADEON_MAC_POWERBOOK_DL      = 0x00000002,
+       RADEON_MAC_POWERBOOK         = 0x00000004,
+       RADEON_MAC_MINI              = 0x00000008
+} RADEONMacModel;
+#endif
+
 typedef enum {
 	CARD_PCI,
 	CARD_AGP,
@@ -776,9 +788,6 @@ typedef struct {
 
     DisplayModePtr currentMode, savedCurrentMode;
 
-    int			constantDPI; /* -1 = auto, 0 = off, 1 = on */
-    int			RADEONDPIVX, RADEONDPIVY;
-
     /* special handlings for DELL triple-head server */
     Bool		IsDellServer; 
 
@@ -799,6 +808,11 @@ typedef struct {
     Bool              crtc2_on;
 
     Bool              InternalTVOut;
+    int               tvdac_use_count;
+
+#if defined(__powerpc__)
+    RADEONMacModel    MacModel;
+#endif
 
     Rotation rotation;
     void (*PointerMoved)(int, int, int);
