@@ -95,7 +95,6 @@ static struct blendinfo RadeonBlendOp[] = {
 
 struct formatinfo {
     int fmt;
-    Bool byte_swap;
     CARD32 card_fmt;
 };
 
@@ -103,25 +102,23 @@ struct formatinfo {
  * TXFORMAT_Y8 expands to (Y,Y,Y,1).  TXFORMAT_I8 expands to (I,I,I,I)
  */
 static struct formatinfo R100TexFormats[] = {
-	{PICT_a8r8g8b8,	0, RADEON_TXFORMAT_ARGB8888 | RADEON_TXFORMAT_ALPHA_IN_MAP},
-	{PICT_x8r8g8b8,	0, RADEON_TXFORMAT_ARGB8888},
-	{PICT_a8b8g8r8,	1, RADEON_TXFORMAT_RGBA8888 | RADEON_TXFORMAT_ALPHA_IN_MAP},
-	{PICT_x8b8g8r8,	1, RADEON_TXFORMAT_RGBA8888},
-	{PICT_r5g6b5,	0, RADEON_TXFORMAT_RGB565},
-	{PICT_a1r5g5b5,	0, RADEON_TXFORMAT_ARGB1555 | RADEON_TXFORMAT_ALPHA_IN_MAP},
-	{PICT_x1r5g5b5,	0, RADEON_TXFORMAT_ARGB1555},
-	{PICT_a8,	0, RADEON_TXFORMAT_I8 | RADEON_TXFORMAT_ALPHA_IN_MAP},
+	{PICT_a8r8g8b8,	RADEON_TXFORMAT_ARGB8888 | RADEON_TXFORMAT_ALPHA_IN_MAP},
+	{PICT_x8r8g8b8,	RADEON_TXFORMAT_ARGB8888},
+	{PICT_r5g6b5,	RADEON_TXFORMAT_RGB565},
+	{PICT_a1r5g5b5,	RADEON_TXFORMAT_ARGB1555 | RADEON_TXFORMAT_ALPHA_IN_MAP},
+	{PICT_x1r5g5b5,	RADEON_TXFORMAT_ARGB1555},
+	{PICT_a8,	RADEON_TXFORMAT_I8 | RADEON_TXFORMAT_ALPHA_IN_MAP},
 };
 
 static struct formatinfo R200TexFormats[] = {
-    {PICT_a8r8g8b8,	0, R200_TXFORMAT_ARGB8888 | R200_TXFORMAT_ALPHA_IN_MAP},
-    {PICT_x8r8g8b8,	0, R200_TXFORMAT_ARGB8888},
-    {PICT_a8b8g8r8,	1, R200_TXFORMAT_RGBA8888 | R200_TXFORMAT_ALPHA_IN_MAP},
-    {PICT_x8b8g8r8,	1, R200_TXFORMAT_RGBA8888},
-    {PICT_r5g6b5,	0, R200_TXFORMAT_RGB565},
-    {PICT_a1r5g5b5,	0, R200_TXFORMAT_ARGB1555 | R200_TXFORMAT_ALPHA_IN_MAP},
-    {PICT_x1r5g5b5,	0, R200_TXFORMAT_ARGB1555},
-    {PICT_a8,		0, R200_TXFORMAT_I8 | R200_TXFORMAT_ALPHA_IN_MAP},
+    {PICT_a8r8g8b8,	R200_TXFORMAT_ARGB8888 | R200_TXFORMAT_ALPHA_IN_MAP},
+    {PICT_x8r8g8b8,	R200_TXFORMAT_ARGB8888},
+    {PICT_a8b8g8r8,	R200_TXFORMAT_ABGR8888 | R200_TXFORMAT_ALPHA_IN_MAP},
+    {PICT_x8b8g8r8,	R200_TXFORMAT_ABGR8888},
+    {PICT_r5g6b5,	R200_TXFORMAT_RGB565},
+    {PICT_a1r5g5b5,	R200_TXFORMAT_ARGB1555 | R200_TXFORMAT_ALPHA_IN_MAP},
+    {PICT_x1r5g5b5,	R200_TXFORMAT_ARGB1555},
+    {PICT_a8,		R200_TXFORMAT_I8 | R200_TXFORMAT_ALPHA_IN_MAP},
 };
 
 /* Common Radeon setup code */
@@ -245,18 +242,24 @@ static Bool FUNC_NAME(R100TextureSetup)(PicturePtr pPict, PixmapPtr pPix,
 	    break;
     }
     txformat = R100TexFormats[i].card_fmt;
-    if (R100TexFormats[i].byte_swap)
-	txoffset |= RADEON_TXO_ENDIAN_BYTE_SWAP;
     if (RADEONPixmapIsColortiled(pPix))
 	txoffset |= RADEON_TXO_MACRO_TILE;
 
     if (pPict->repeat) {
+	if ((h != 1) &&
+	    (((w * pPix->drawable.bitsPerPixel / 8 + 31) & ~31) != txpitch))
+	    RADEON_FALLBACK(("Width %d and pitch %u not compatible for repeat\n",
+			     w, (unsigned)txpitch));
+
 	txformat |= RADEONLog2(w) << RADEON_TXFORMAT_WIDTH_SHIFT;
 	txformat |= RADEONLog2(h) << RADEON_TXFORMAT_HEIGHT_SHIFT;
-    } else 
+    } else
 	txformat |= RADEON_TXFORMAT_NON_POWER2;
     txformat |= unit << 24; /* RADEON_TXFORMAT_ST_ROUTE_STQX */
- 
+
+    info->texW[unit] = 1;
+    info->texH[unit] = 1;
+
     switch (pPict->filter) {
     case PictFilterNearest:
 	txfilter = (RADEON_MAG_FILTER_NEAREST | RADEON_MIN_FILTER_NEAREST);
@@ -500,18 +503,24 @@ static Bool FUNC_NAME(R200TextureSetup)(PicturePtr pPict, PixmapPtr pPix,
 	    break;
     }
     txformat = R200TexFormats[i].card_fmt;
-    if (R200TexFormats[i].byte_swap)
-	txoffset |= R200_TXO_ENDIAN_BYTE_SWAP;
     if (RADEONPixmapIsColortiled(pPix))
 	txoffset |= R200_TXO_MACRO_TILE;
 
     if (pPict->repeat) {
+	if ((h != 1) &&
+	    (((w * pPix->drawable.bitsPerPixel / 8 + 31) & ~31) != txpitch))
+	    RADEON_FALLBACK(("Width %d and pitch %u not compatible for repeat\n",
+			     w, (unsigned)txpitch));
+
 	txformat |= RADEONLog2(w) << R200_TXFORMAT_WIDTH_SHIFT;
 	txformat |= RADEONLog2(h) << R200_TXFORMAT_HEIGHT_SHIFT;
     } else
 	txformat |= R200_TXFORMAT_NON_POWER2;
     txformat |= unit << R200_TXFORMAT_ST_ROUTE_SHIFT;
 
+    info->texW[unit] = w;
+    info->texH[unit] = h;
+
     switch (pPict->filter) {
     case PictFilterNearest:
 	txfilter = (R200_MAG_FILTER_NEAREST |
@@ -798,11 +807,6 @@ static void FUNC_NAME(RadeonComposite)(PixmapPtr pDst,
 		 (4 << RADEON_CP_VC_CNTL_NUM_SHIFT));
     }
 
-    VTX_OUT(dstX,     dstY,     srcX,    srcY,    maskX,    maskY);
-    VTX_OUT(dstX,     dstY + h, srcX,    srcYend, maskX,    maskYend);
-    VTX_OUT(dstX + w, dstY + h, srcXend, srcYend, maskXend, maskYend);
-    VTX_OUT(dstX + w, dstY,     srcXend, srcY,    maskXend, maskY);
-    ADVANCE_RING();
 #else /* ACCEL_CP */
     BEGIN_ACCEL(1 + VTX_REG_COUNT * 4);
     if (info->ChipFamily < CHIP_FAMILY_R200) {
@@ -815,11 +819,32 @@ static void FUNC_NAME(RadeonComposite)(PixmapPtr pDst,
 					  RADEON_VF_PRIM_WALK_DATA |
 					  4 << RADEON_VF_NUM_VERTICES_SHIFT));
     }
+#endif
 
-    VTX_OUT(dstX,     dstY,     srcX,    srcY,    maskX,    maskY);
-    VTX_OUT(dstX,     dstY + h, srcX,    srcYend, maskX,    maskYend);
-    VTX_OUT(dstX + w, dstY + h, srcXend, srcYend, maskXend, maskYend);
-    VTX_OUT(dstX + w, dstY,     srcXend, srcY,    maskXend, maskY);
+    if (info->texW[0] == 1 && info->texH[0] == 1 &&
+	info->texW[1] == 1 && info->texH[1] == 1) {
+	VTX_OUT(dstX,     dstY,       srcX,     srcY,	  maskX,    maskY);
+	VTX_OUT(dstX,     dstY + h,   srcX,     srcYend,  maskX,    maskYend);
+	VTX_OUT(dstX + w, dstY + h,   srcXend,  srcYend,  maskXend, maskYend);
+	VTX_OUT(dstX + w, dstY,	      srcXend,  srcY,     maskXend, maskY);
+    } else {
+    VTX_OUT((float)dstX,     (float)dstY,
+	    (float)srcX / info->texW[0],     (float)srcY / info->texH[0],
+	    (float)maskX / info->texW[1],    (float)maskY / info->texH[1]);
+    VTX_OUT((float)dstX,     (float)(dstY + h),
+	    (float)srcX / info->texW[0],     (float)srcYend / info->texH[0],
+	    (float)maskX / info->texW[1],    (float)maskYend / info->texH[1]);
+    VTX_OUT((float)(dstX + w), (float)(dstY + h),
+	    (float)srcXend / info->texW[0],  (float)srcYend / info->texH[0],
+	    (float)maskXend / info->texW[1], (float)maskYend / info->texH[1]);
+    VTX_OUT((float)(dstX + w), (float)dstY,
+	    (float)srcXend / info->texW[0],  (float)srcY / info->texH[0],
+	    (float)maskXend / info->texW[1], (float)maskY / info->texH[1]);
+    }
+
+#ifdef ACCEL_CP
+    ADVANCE_RING();
+#else
     FINISH_ACCEL();
 #endif /* !ACCEL_CP */
 
